package server.data;

import exception.DatabaseException;
import exception.ParserException;
import server.database.schema.Schema;
import server.database.schema.maintenance.PriorUserTableSchema;
import server.database.sql.SQLDatabase;
import server.database.sql.SQLSelect;
import server.database.sql.SQLTable;
import server.parser.Formula;

public class PriorUser extends IdAndFormulaTable {

	PriorUser(SQLDatabase db, int userId) throws DatabaseException {
		super(db, Schema.maintenanceDatabase.priorUser, userId);
		
		// Load entries from database.
		SQLSelect sql = this.db.newSelect();
		sql.select(PriorUserTableSchema.ID).select(PriorUserTableSchema.FORMULA);
		sql.from(Schema.maintenanceDatabase.priorUser);
		
		SQLTable result = sql.get();
		for ( String[] row : result ) {
			try {
				int id = Integer.valueOf( row[0] );
				Formula formula = new Formula( row[1] );
				
				// Create new IdAndFormulaEntry and add to list.
				IdAndFormulaEntry entry = new IdAndFormulaEntry(this, id, formula);
				this.entries.add( entry );
			} catch (ParserException e) {
				throw new DatabaseException("Unable to parse prior user entry (malformed formula)", e);
			} catch (NumberFormatException e) {
				throw new DatabaseException("Unable to parse prior user entry (malformed id)", e);
			}
		}
	}
}
